# -*- coding:utf-8 -*-

from ...errors.saklientexception import SaklientException
from ..client import Client
from .resource import Resource
from .icon import Icon
from .ipv4net import Ipv4Net
from .ipv6net import Ipv6Net
from ...util import Util

# module saklient.cloud.resources.router

class Router(Resource):
    ## ルータの実体1つに対応し、属性の取得や操作を行うためのクラス。
    
    # (instance field) m_id
    
    # (instance field) m_name
    
    # (instance field) m_description
    
    # (instance field) m_network_mask_len
    
    # (instance field) m_band_width_mbps
    
    # (instance field) m_swytch_id
    
    ## @private
    # @return {str}
    def _api_path(self):
        return "/internet"
    
    ## @private
    # @return {str}
    def _root_key(self):
        return "Internet"
    
    ## @private
    # @return {str}
    def _root_key_m(self):
        return "Internet"
    
    ## @private
    # @return {str}
    def _class_name(self):
        return "Router"
    
    ## @private
    # @return {str}
    def _id(self):
        return self.get_id()
    
    ## このローカルオブジェクトに現在設定されているリソース情報をAPIに送信し、新規作成または上書き保存します。
    # 
    # @return {saklient.cloud.resources.router.Router} this
    def save(self):
        return self._save()
    
    ## 最新のリソース情報を再取得します。
    # 
    # @return {saklient.cloud.resources.router.Router} this
    def reload(self):
        return self._reload()
    
    ## @ignore
    # @param {saklient.cloud.client.Client} client
    # @param {any} obj
    # @param {bool} wrapped=False
    def __init__(self, client, obj, wrapped=False):
        super(Router, self).__init__(client)
        Util.validate_type(client, "saklient.cloud.client.Client")
        Util.validate_type(wrapped, "bool")
        self.api_deserialize(obj, wrapped)
    
    ## 作成中のルータが利用可能になるまで待機します。
    # 
    # @ignore
    # @param {int} timeoutSec
    # @param {(saklient.cloud.resources.router.Router, bool) => void} callback
    # @return {void}
    def after_create(self, timeoutSec, callback):
        Util.validate_type(timeoutSec, "int")
        Util.validate_type(callback, "function")
        ret = self.sleep_while_creating(timeoutSec)
        callback(self, ret)
    
    ## 作成中のルータが利用可能になるまで待機します。
    # 
    # @param {int} timeoutSec=120
    # @return {bool} 成功時はtrue、タイムアウトやエラーによる失敗時はfalseを返します。
    def sleep_while_creating(self, timeoutSec=120):
        Util.validate_type(timeoutSec, "int")
        step = 3
        while (0 < timeoutSec):
            if self.exists():
                self.reload()
                return True
            timeoutSec -= step
            if 0 < timeoutSec:
                Util.sleep(step)
        return False
    
    ## このルータが接続されているスイッチを取得します。
    # 
    # @return {Swytch}
    def get_swytch(self):
        model = Util.create_class_instance("saklient.cloud.models.Model_Swytch", [self._client])
        id = self.get_swytch_id()
        return model.get_by_id(id)
    
    ## このルータ＋スイッチでIPv6アドレスを有効にします。
    # 
    # @return {saklient.cloud.resources.ipv6net.Ipv6Net} 有効化されたIPv6ネットワーク
    def add_ipv6_net(self):
        result = self._client.request("POST", self._api_path() + "/" + Util.url_encode(self._id()) + "/ipv6net")
        self.reload()
        return Ipv6Net(self._client, (result["IPv6Net"] if "IPv6Net" in result else None))
    
    ## このルータ＋スイッチでIPv6アドレスを無効にします。
    # 
    # @param {saklient.cloud.resources.ipv6net.Ipv6Net} ipv6Net
    # @return {saklient.cloud.resources.router.Router} this
    def remove_ipv6_net(self, ipv6Net):
        Util.validate_type(ipv6Net, "saklient.cloud.resources.ipv6net.Ipv6Net")
        self._client.request("DELETE", self._api_path() + "/" + Util.url_encode(self._id()) + "/ipv6net/" + ipv6Net._id())
        self.reload()
        return self
    
    ## このルータ＋スイッチにスタティックルートを追加します。
    # 
    # @param {int} maskLen
    # @param {str} nextHop
    # @return {saklient.cloud.resources.ipv4net.Ipv4Net} 追加されたスタティックルート
    def add_static_route(self, maskLen, nextHop):
        Util.validate_type(maskLen, "int")
        Util.validate_type(nextHop, "str")
        q = {}
        Util.set_by_path(q, "NetworkMaskLen", maskLen)
        Util.set_by_path(q, "NextHop", nextHop)
        result = self._client.request("POST", self._api_path() + "/" + Util.url_encode(self._id()) + "/subnet", q)
        self.reload()
        return Ipv4Net(self._client, (result["Subnet"] if "Subnet" in result else None))
    
    ## このルータ＋スイッチからスタティックルートを削除します。
    # 
    # @param {saklient.cloud.resources.ipv4net.Ipv4Net} ipv4Net
    # @return {saklient.cloud.resources.router.Router} this
    def remove_static_route(self, ipv4Net):
        Util.validate_type(ipv4Net, "saklient.cloud.resources.ipv4net.Ipv4Net")
        self._client.request("DELETE", self._api_path() + "/" + Util.url_encode(self._id()) + "/subnet/" + ipv4Net._id())
        self.reload()
        return self
    
    ## このルータ＋スイッチの帯域プランを変更します。
    # 
    # 成功時はリソースIDが変わることにご注意ください。
    # 
    # @param {int} bandWidthMbps
    # @return {saklient.cloud.resources.router.Router} this
    def change_plan(self, bandWidthMbps):
        Util.validate_type(bandWidthMbps, "int")
        path = self._api_path() + "/" + Util.url_encode(self._id()) + "/bandwidth"
        q = {}
        Util.set_by_path(q, "Internet.BandWidthMbps", bandWidthMbps)
        result = self._client.request("PUT", path, q)
        self.api_deserialize(result, True)
        return self
    
    # (instance field) n_id = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_id(self):
        return self.m_id
    
    ## ID
    id = property(get_id, None, None)
    
    # (instance field) n_name = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_name(self):
        return self.m_name
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {str} v
    # @return {str}
    def set_name(self, v):
        Util.validate_type(v, "str")
        self.m_name = v
        self.n_name = True
        return self.m_name
    
    ## 名前
    name = property(get_name, set_name, None)
    
    # (instance field) n_description = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_description(self):
        return self.m_description
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {str} v
    # @return {str}
    def set_description(self, v):
        Util.validate_type(v, "str")
        self.m_description = v
        self.n_description = True
        return self.m_description
    
    ## 説明
    description = property(get_description, set_description, None)
    
    # (instance field) n_network_mask_len = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {int}
    def get_network_mask_len(self):
        return self.m_network_mask_len
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {int} v
    # @return {int}
    def set_network_mask_len(self, v):
        Util.validate_type(v, "int")
        if not self.is_new:
            raise SaklientException("immutable_field", "Immutable fields cannot be modified after the resource creation: " + "saklient.cloud.resources.router.Router#network_mask_len")
        self.m_network_mask_len = v
        self.n_network_mask_len = True
        return self.m_network_mask_len
    
    ## ネットワークのマスク長
    network_mask_len = property(get_network_mask_len, set_network_mask_len, None)
    
    # (instance field) n_band_width_mbps = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {int}
    def get_band_width_mbps(self):
        return self.m_band_width_mbps
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {int} v
    # @return {int}
    def set_band_width_mbps(self, v):
        Util.validate_type(v, "int")
        if not self.is_new:
            raise SaklientException("immutable_field", "Immutable fields cannot be modified after the resource creation: " + "saklient.cloud.resources.router.Router#band_width_mbps")
        self.m_band_width_mbps = v
        self.n_band_width_mbps = True
        return self.m_band_width_mbps
    
    ## 帯域幅
    band_width_mbps = property(get_band_width_mbps, set_band_width_mbps, None)
    
    # (instance field) n_swytch_id = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_swytch_id(self):
        return self.m_swytch_id
    
    ## スイッチ
    swytch_id = property(get_swytch_id, None, None)
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {any} r
    def api_deserialize_impl(self, r):
        self.is_new = r is None
        if self.is_new:
            r = {
                
            }
        self.is_incomplete = False
        if Util.exists_path(r, "ID"):
            self.m_id = None if Util.get_by_path(r, "ID") is None else str(Util.get_by_path(r, "ID"))
        else:
            self.m_id = None
            self.is_incomplete = True
        self.n_id = False
        if Util.exists_path(r, "Name"):
            self.m_name = None if Util.get_by_path(r, "Name") is None else str(Util.get_by_path(r, "Name"))
        else:
            self.m_name = None
            self.is_incomplete = True
        self.n_name = False
        if Util.exists_path(r, "Description"):
            self.m_description = None if Util.get_by_path(r, "Description") is None else str(Util.get_by_path(r, "Description"))
        else:
            self.m_description = None
            self.is_incomplete = True
        self.n_description = False
        if Util.exists_path(r, "NetworkMaskLen"):
            self.m_network_mask_len = None if Util.get_by_path(r, "NetworkMaskLen") is None else int(str(Util.get_by_path(r, "NetworkMaskLen")))
        else:
            self.m_network_mask_len = None
            self.is_incomplete = True
        self.n_network_mask_len = False
        if Util.exists_path(r, "BandWidthMbps"):
            self.m_band_width_mbps = None if Util.get_by_path(r, "BandWidthMbps") is None else int(str(Util.get_by_path(r, "BandWidthMbps")))
        else:
            self.m_band_width_mbps = None
            self.is_incomplete = True
        self.n_band_width_mbps = False
        if Util.exists_path(r, "Switch.ID"):
            self.m_swytch_id = None if Util.get_by_path(r, "Switch.ID") is None else str(Util.get_by_path(r, "Switch.ID"))
        else:
            self.m_swytch_id = None
            self.is_incomplete = True
        self.n_swytch_id = False
    
    ## @ignore
    # @param {bool} withClean=False
    # @return {any}
    def api_serialize_impl(self, withClean=False):
        Util.validate_type(withClean, "bool")
        missing = []
        ret = {
            
        }
        if withClean or self.n_id:
            Util.set_by_path(ret, "ID", self.m_id)
        if withClean or self.n_name:
            Util.set_by_path(ret, "Name", self.m_name)
        else:
            if self.is_new:
                missing.append("name")
        if withClean or self.n_description:
            Util.set_by_path(ret, "Description", self.m_description)
        if withClean or self.n_network_mask_len:
            Util.set_by_path(ret, "NetworkMaskLen", self.m_network_mask_len)
        else:
            if self.is_new:
                missing.append("network_mask_len")
        if withClean or self.n_band_width_mbps:
            Util.set_by_path(ret, "BandWidthMbps", self.m_band_width_mbps)
        else:
            if self.is_new:
                missing.append("band_width_mbps")
        if withClean or self.n_swytch_id:
            Util.set_by_path(ret, "Switch.ID", self.m_swytch_id)
        if len(missing) > 0:
            raise SaklientException("required_field", "Required fields must be set before the Router creation: " + ", ".join(missing))
        return ret
    



# -*- coding:utf-8 -*-

from ...errors.saklientexception import SaklientException
from ..client import Client
from .appliance import Appliance
from .lbvirtualip import LbVirtualIp
from .swytch import Swytch
from .ipv4net import Ipv4Net
from ..enums.eapplianceclass import EApplianceClass
from ...util import Util

# module saklient.cloud.resources.loadbalancer

class LoadBalancer(Appliance):
    ## ロードバランサの実体1つに対応し、属性の取得や操作を行うためのクラス。
    
    # (instance field) _virtual_ips
    
    ## @return {saklient.cloud.resources.lbvirtualip.LbVirtualIp[]}
    def get_virtual_ips(self):
        return self._virtual_ips
    
    ## 仮想IPアドレス {@link LbVirtualIp} の配列
    virtual_ips = property(get_virtual_ips, None, None)
    
    ## @return {str}
    def get_default_route(self):
        return Util.get_by_path(self.raw_annotation, "Network.DefaultRoute")
    
    ## @param {str} v
    # @return {str}
    def set_default_route(self, v):
        Util.validate_type(v, "str")
        Util.set_by_path(self.raw_annotation, "Network.DefaultRoute", v)
        return v
    
    ## デフォルトルート
    default_route = property(get_default_route, set_default_route, None)
    
    ## @return {int}
    def get_mask_len(self):
        maskLen = Util.get_by_path(self.raw_annotation, "Network.NetworkMaskLen")
        if maskLen is None:
            raise SaklientException("invalid_data", "Data of the resource is invalid")
        return int(maskLen)
    
    ## @param {int} v
    # @return {int}
    def set_mask_len(self, v):
        Util.validate_type(v, "int")
        Util.set_by_path(self.raw_annotation, "Network.NetworkMaskLen", v)
        return v
    
    ## マスク長
    mask_len = property(get_mask_len, set_mask_len, None)
    
    ## @return {int}
    def get_vrid(self):
        vrid = Util.get_by_path(self.raw_annotation, "VRRP.VRID")
        if vrid is None:
            raise SaklientException("invalid_data", "Data of the resource is invalid")
        return int(vrid)
    
    ## @param {int} v
    # @return {int}
    def set_vrid(self, v):
        Util.validate_type(v, "int")
        Util.set_by_path(self.raw_annotation, "VRRP.VRID", v)
        return v
    
    ## VRID
    vrid = property(get_vrid, set_vrid, None)
    
    ## @ignore
    # @param {saklient.cloud.client.Client} client
    # @param {any} obj
    # @param {bool} wrapped=False
    def __init__(self, client, obj, wrapped=False):
        super(LoadBalancer, self).__init__(client, obj, wrapped)
        Util.validate_type(client, "saklient.cloud.client.Client")
        Util.validate_type(wrapped, "bool")
        if self.raw_annotation is None:
            self.raw_annotation = {}
    
    ## @private
    # @param {any} r
    # @param {any} root
    # @return {void}
    def _on_after_api_deserialize(self, r, root):
        if self.raw_annotation is None:
            self.raw_annotation = {}
        self._virtual_ips = []
        settings = self.raw_settings
        if settings is not None:
            lb = (settings["LoadBalancer"] if "LoadBalancer" in settings else None)
            if lb is None:
                lb = []
            vips = lb
            for vip in vips:
                self._virtual_ips.append(LbVirtualIp(vip))
    
    ## @private
    # @param {bool} withClean
    # @return {void}
    def _on_before_api_serialize(self, withClean):
        Util.validate_type(withClean, "bool")
        lb = []
        for vip in self._virtual_ips:
            lb.append(vip.to_raw_settings())
        if self.raw_settings is None:
            self.raw_settings = {}
        self.raw_settings["LoadBalancer"] = lb
        if self.is_new:
            self.clazz = EApplianceClass.loadbalancer
    
    ## @ignore
    # @param {saklient.cloud.resources.swytch.Swytch} swytch
    # @param {int} vrid
    # @param {str[]} realIps
    # @param {bool} isHighSpec=False
    # @return {saklient.cloud.resources.loadbalancer.LoadBalancer}
    def set_initial_params(self, swytch, vrid, realIps, isHighSpec=False):
        Util.validate_type(swytch, "saklient.cloud.resources.swytch.Swytch")
        Util.validate_type(vrid, "int")
        Util.validate_type(realIps, "list")
        Util.validate_type(isHighSpec, "bool")
        annot = self.raw_annotation
        self.vrid = vrid
        Util.set_by_path(annot, "Switch.ID", swytch._id())
        if swytch.ipv4_nets is not None and 0 < len(swytch.ipv4_nets):
            net = swytch.ipv4_nets[0]
            self.default_route = net.default_route
            self.mask_len = net.mask_len
        else:
            self.default_route = swytch.user_default_route
            self.mask_len = swytch.user_mask_len
        servers = []
        for ip in realIps:
            servers.append({
                'IPAddress': ip
            })
        Util.set_by_path(annot, "Servers", servers)
        self.plan_id = 2 if isHighSpec else 1
        return self
    
    ## @return {saklient.cloud.resources.loadbalancer.LoadBalancer}
    def clear_virtual_ips(self):
        while (0 < len(self._virtual_ips)):
            self._virtual_ips.pop()
        return self
    
    ## @param {any} settings=None
    # @return {saklient.cloud.resources.lbvirtualip.LbVirtualIp}
    def add_virtual_ip(self, settings=None):
        ret = LbVirtualIp(settings)
        self._virtual_ips.append(ret)
        return ret
    
    ## @param {str} address
    # @return {saklient.cloud.resources.lbvirtualip.LbVirtualIp}
    def get_virtual_ip_by_address(self, address):
        Util.validate_type(address, "str")
        for vip in self._virtual_ips:
            if vip.virtual_ip_address == address:
                return vip
        return None
    
    ## @return {saklient.cloud.resources.loadbalancer.LoadBalancer}
    def reload_status(self):
        result = self.request_retry("GET", self._api_path() + "/" + Util.url_encode(self._id()) + "/status")
        vips = (result["LoadBalancer"] if "LoadBalancer" in result else None)
        for vipDyn in vips:
            vipStr = (vipDyn["VirtualIPAddress"] if "VirtualIPAddress" in vipDyn else None)
            vip = self.get_virtual_ip_by_address(vipStr)
            if vip is None:
                next
            vip.update_status((vipDyn["Servers"] if "Servers" in vipDyn else None))
        return self
    



# -*- coding:utf-8 -*-

from ..client import Client
from .resource import Resource
from ...util import Util

# module saklient.cloud.resources.ipv6net

class Ipv6Net(Resource):
    ## IPv6ネットワークの実体1つに対応し、属性の取得や操作を行うためのクラス。
    
    # (instance field) m_id
    
    # (instance field) m_prefix
    
    # (instance field) m_prefix_len
    
    # (instance field) m_prefix_tail
    
    ## @private
    # @return {str}
    def _api_path(self):
        return "/ipv6net"
    
    ## @private
    # @return {str}
    def _root_key(self):
        return "IPv6Net"
    
    ## @private
    # @return {str}
    def _root_key_m(self):
        return "IPv6Nets"
    
    ## @private
    # @return {str}
    def _class_name(self):
        return "Ipv6Net"
    
    ## @private
    # @return {str}
    def _id(self):
        return self.get_id()
    
    ## 最新のリソース情報を再取得します。
    # 
    # @return {Swytch} this
    def reload(self):
        return self._reload()
    
    ## @ignore
    # @param {saklient.cloud.client.Client} client
    # @param {any} obj
    # @param {bool} wrapped=False
    def __init__(self, client, obj, wrapped=False):
        super(Ipv6Net, self).__init__(client)
        Util.validate_type(client, "saklient.cloud.client.Client")
        Util.validate_type(wrapped, "bool")
        self.api_deserialize(obj, wrapped)
    
    # (instance field) n_id = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_id(self):
        return self.m_id
    
    ## ID
    id = property(get_id, None, None)
    
    # (instance field) n_prefix = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_prefix(self):
        return self.m_prefix
    
    ## ネットワークプレフィックス
    prefix = property(get_prefix, None, None)
    
    # (instance field) n_prefix_len = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {int}
    def get_prefix_len(self):
        return self.m_prefix_len
    
    ## ネットワークプレフィックス長
    prefix_len = property(get_prefix_len, None, None)
    
    # (instance field) n_prefix_tail = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_prefix_tail(self):
        return self.m_prefix_tail
    
    ## このネットワーク範囲における最後のIPv6アドレス
    prefix_tail = property(get_prefix_tail, None, None)
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {any} r
    def api_deserialize_impl(self, r):
        self.is_new = r is None
        if self.is_new:
            r = {
                
            }
        self.is_incomplete = False
        if Util.exists_path(r, "ID"):
            self.m_id = None if Util.get_by_path(r, "ID") is None else str(Util.get_by_path(r, "ID"))
        else:
            self.m_id = None
            self.is_incomplete = True
        self.n_id = False
        if Util.exists_path(r, "IPv6Prefix"):
            self.m_prefix = None if Util.get_by_path(r, "IPv6Prefix") is None else str(Util.get_by_path(r, "IPv6Prefix"))
        else:
            self.m_prefix = None
            self.is_incomplete = True
        self.n_prefix = False
        if Util.exists_path(r, "IPv6PrefixLen"):
            self.m_prefix_len = None if Util.get_by_path(r, "IPv6PrefixLen") is None else int(str(Util.get_by_path(r, "IPv6PrefixLen")))
        else:
            self.m_prefix_len = None
            self.is_incomplete = True
        self.n_prefix_len = False
        if Util.exists_path(r, "IPv6PrefixTail"):
            self.m_prefix_tail = None if Util.get_by_path(r, "IPv6PrefixTail") is None else str(Util.get_by_path(r, "IPv6PrefixTail"))
        else:
            self.m_prefix_tail = None
            self.is_incomplete = True
        self.n_prefix_tail = False
    
    ## @ignore
    # @param {bool} withClean=False
    # @return {any}
    def api_serialize_impl(self, withClean=False):
        Util.validate_type(withClean, "bool")
        ret = {
            
        }
        if withClean or self.n_id:
            Util.set_by_path(ret, "ID", self.m_id)
        if withClean or self.n_prefix:
            Util.set_by_path(ret, "IPv6Prefix", self.m_prefix)
        if withClean or self.n_prefix_len:
            Util.set_by_path(ret, "IPv6PrefixLen", self.m_prefix_len)
        if withClean or self.n_prefix_tail:
            Util.set_by_path(ret, "IPv6PrefixTail", self.m_prefix_tail)
        return ret
    



# -*- coding:utf-8 -*-

from ...errors.httpforbiddenexception import HttpForbiddenException

# module saklient.cloud.errors.accountnotspecifiedexception

class AccountNotSpecifiedException(HttpForbiddenException):
    ## 要求された操作は許可されていません。このAPIはアカウントを特定できる認証方法でアクセスする必要があります。
    
    ## @param {int} status
    # @param {str} code=None
    # @param {str} message=""
    def __init__(self, status, code=None, message=""):
        super(AccountNotSpecifiedException, self).__init__(status, code, "要求された操作は許可されていません。このAPIはアカウントを特定できる認証方法でアクセスする必要があります。" if message is None or message == "" else message)
    



# -*- coding:utf-8 -*-

from ...util import Util

# module saklient.cloud.enums.estorageclass

class EStorageClass:
    ## ストレージのクラスを表す列挙子。
    
    ISCSI1204 = "iscsi1204"
    iscsi1204 = "iscsi1204"
    
    ## @ignore
    _MAP = {
        "ISCSI1204": 110
    }
    
    ## @ignore
    @classmethod
    def compare(clazz, lhs, rhs):
        if not (isinstance(lhs, str) and isinstance(rhs, str)): return None
        lhs = lhs.upper()
        rhs = rhs.upper()
        if lhs not in clazz._MAP or rhs not in clazz._MAP: return None
        d = clazz._MAP[lhs] - clazz._MAP[rhs]
        return (0<d) - (d<0)
    
    



# -*- coding:utf-8 -*-

from .httpexception import HttpException

# module saklient.errors.httpunauthorizedexception

class HttpUnauthorizedException(HttpException):
    ## この操作は認証が必要です。IDまたはパスワードが誤っている可能性があります。
    
    ## @param {int} status
    # @param {str} code=None
    # @param {str} message=""
    def __init__(self, status, code=None, message=""):
        super(HttpUnauthorizedException, self).__init__(status, code, "この操作は認証が必要です。IDまたはパスワードが誤っている可能性があります。" if message is None or message == "" else message)
    



# -*- coding:utf-8 -*-

from ..client import Client
from .resource import Resource
from ...util import Util

# module saklient.cloud.resources.routerplan

class RouterPlan(Resource):
    ## ルータ帯域プラン情報の1レコードに対応するクラス。
    
    # (instance field) m_id
    
    # (instance field) m_name
    
    # (instance field) m_band_width_mbps
    
    # (instance field) m_service_class
    
    ## @private
    # @return {str}
    def _api_path(self):
        return "/product/internet"
    
    ## @private
    # @return {str}
    def _root_key(self):
        return "InternetPlan"
    
    ## @private
    # @return {str}
    def _root_key_m(self):
        return "InternetPlans"
    
    ## @private
    # @return {str}
    def _class_name(self):
        return "RouterPlan"
    
    ## @private
    # @return {str}
    def _id(self):
        return self.get_id()
    
    ## @ignore
    # @param {saklient.cloud.client.Client} client
    # @param {any} obj
    # @param {bool} wrapped=False
    def __init__(self, client, obj, wrapped=False):
        super(RouterPlan, self).__init__(client)
        Util.validate_type(client, "saklient.cloud.client.Client")
        Util.validate_type(wrapped, "bool")
        self.api_deserialize(obj, wrapped)
    
    # (instance field) n_id = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_id(self):
        return self.m_id
    
    ## ID
    id = property(get_id, None, None)
    
    # (instance field) n_name = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_name(self):
        return self.m_name
    
    ## 名前
    name = property(get_name, None, None)
    
    # (instance field) n_band_width_mbps = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {int}
    def get_band_width_mbps(self):
        return self.m_band_width_mbps
    
    ## 帯域幅
    band_width_mbps = property(get_band_width_mbps, None, None)
    
    # (instance field) n_service_class = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_service_class(self):
        return self.m_service_class
    
    ## サービスクラス
    service_class = property(get_service_class, None, None)
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {any} r
    def api_deserialize_impl(self, r):
        self.is_new = r is None
        if self.is_new:
            r = {
                
            }
        self.is_incomplete = False
        if Util.exists_path(r, "ID"):
            self.m_id = None if Util.get_by_path(r, "ID") is None else str(Util.get_by_path(r, "ID"))
        else:
            self.m_id = None
            self.is_incomplete = True
        self.n_id = False
        if Util.exists_path(r, "Name"):
            self.m_name = None if Util.get_by_path(r, "Name") is None else str(Util.get_by_path(r, "Name"))
        else:
            self.m_name = None
            self.is_incomplete = True
        self.n_name = False
        if Util.exists_path(r, "BandWidthMbps"):
            self.m_band_width_mbps = None if Util.get_by_path(r, "BandWidthMbps") is None else int(str(Util.get_by_path(r, "BandWidthMbps")))
        else:
            self.m_band_width_mbps = None
            self.is_incomplete = True
        self.n_band_width_mbps = False
        if Util.exists_path(r, "ServiceClass"):
            self.m_service_class = None if Util.get_by_path(r, "ServiceClass") is None else str(Util.get_by_path(r, "ServiceClass"))
        else:
            self.m_service_class = None
            self.is_incomplete = True
        self.n_service_class = False
    
    ## @ignore
    # @param {bool} withClean=False
    # @return {any}
    def api_serialize_impl(self, withClean=False):
        Util.validate_type(withClean, "bool")
        ret = {
            
        }
        if withClean or self.n_id:
            Util.set_by_path(ret, "ID", self.m_id)
        if withClean or self.n_name:
            Util.set_by_path(ret, "Name", self.m_name)
        if withClean or self.n_band_width_mbps:
            Util.set_by_path(ret, "BandWidthMbps", self.m_band_width_mbps)
        if withClean or self.n_service_class:
            Util.set_by_path(ret, "ServiceClass", self.m_service_class)
        return ret
    



# -*- coding:utf-8 -*-

from ...errors.httpforbiddenexception import HttpForbiddenException

# module saklient.cloud.errors.accessxhrorapikeyexception

class AccessXhrOrApiKeyException(HttpForbiddenException):
    ## 要求された操作は許可されていません。XHRまたはAPIキーによるアクセスのみ許可されています。
    
    ## @param {int} status
    # @param {str} code=None
    # @param {str} message=""
    def __init__(self, status, code=None, message=""):
        super(AccessXhrOrApiKeyException, self).__init__(status, code, "要求された操作は許可されていません。XHRまたはAPIキーによるアクセスのみ許可されています。" if message is None or message == "" else message)
    



# -*- coding:utf-8 -*-

from ...errors.httpbadrequestexception import HttpBadRequestException

# module saklient.cloud.errors.mustbeofsamezoneexception

class MustBeOfSameZoneException(HttpBadRequestException):
    ## 不適切な要求です。参照するリソースは同一ゾーンに存在しなければなりません。
    
    ## @param {int} status
    # @param {str} code=None
    # @param {str} message=""
    def __init__(self, status, code=None, message=""):
        super(MustBeOfSameZoneException, self).__init__(status, code, "不適切な要求です。参照するリソースは同一ゾーンに存在しなければなりません。" if message is None or message == "" else message)
    



# -*- coding:utf-8 -*-

from ..client import Client
from .model import Model
from ..resources.resource import Resource
from ..resources.disk import Disk
from ...util import Util

# module saklient.cloud.models.model_disk

class Model_Disk(Model):
    ## ディスクを検索・作成するための機能を備えたクラス。
    
    ## @private
    # @return {str}
    def _api_path(self):
        return "/disk"
    
    ## @private
    # @return {str}
    def _root_key(self):
        return "Disk"
    
    ## @private
    # @return {str}
    def _root_key_m(self):
        return "Disks"
    
    ## @private
    # @return {str}
    def _class_name(self):
        return "Disk"
    
    ## @private
    # @param {any} obj
    # @param {bool} wrapped=False
    # @return {saklient.cloud.resources.resource.Resource}
    def _create_resource_impl(self, obj, wrapped=False):
        Util.validate_type(wrapped, "bool")
        return Disk(self._client, obj, wrapped)
    
    ## 次に取得するリストの開始オフセットを指定します。
    # 
    # @param {int} offset オフセット
    # @return {saklient.cloud.models.model_disk.Model_Disk} this
    def offset(self, offset):
        Util.validate_type(offset, "int")
        return self._offset(offset)
    
    ## 次に取得するリストの上限レコード数を指定します。
    # 
    # @param {int} count 上限レコード数
    # @return {saklient.cloud.models.model_disk.Model_Disk} this
    def limit(self, count):
        Util.validate_type(count, "int")
        return self._limit(count)
    
    ## Web APIのフィルタリング設定を直接指定します。
    # 
    # @param {str} key キー
    # @param {any} value 値
    # @param {bool} multiple=False valueに配列を与え、OR条件で完全一致検索する場合にtrueを指定します。通常、valueはスカラ値であいまい検索されます。
    # @return {saklient.cloud.models.model_disk.Model_Disk}
    def filter_by(self, key, value, multiple=False):
        Util.validate_type(key, "str")
        Util.validate_type(multiple, "bool")
        return self._filter_by(key, value, multiple)
    
    ## 次のリクエストのために設定されているステートをすべて破棄します。
    # 
    # @return {saklient.cloud.models.model_disk.Model_Disk} this
    def reset(self):
        return self._reset()
    
    ## 新規リソース作成用のオブジェクトを用意します。
    # 
    # 返り値のオブジェクトにパラメータを設定し、save() を呼ぶことで実際のリソースが作成されます。
    # 
    # @return {saklient.cloud.resources.disk.Disk} リソースオブジェクト
    def create(self):
        return self._create()
    
    ## 指定したIDを持つ唯一のリソースを取得します。
    # 
    # @param {str} id
    # @return {saklient.cloud.resources.disk.Disk} リソースオブジェクト
    def get_by_id(self, id):
        Util.validate_type(id, "str")
        return self._get_by_id(id)
    
    ## リソースの検索リクエストを実行し、結果をリストで取得します。
    # 
    # @return {saklient.cloud.resources.disk.Disk[]} リソースオブジェクトの配列
    def find(self):
        return self._find()
    
    ## 指定した文字列を名前に含むリソースに絞り込みます。
    # 
    # 大文字・小文字は区別されません。
    # 半角スペースで区切られた複数の文字列は、それらをすべて含むことが条件とみなされます。
    # 
    # @todo Implement test case
    # @param {str} name
    # @return {saklient.cloud.models.model_disk.Model_Disk}
    def with_name_like(self, name):
        Util.validate_type(name, "str")
        return self._with_name_like(name)
    
    ## 指定したタグを持つリソースに絞り込みます。
    # 
    # 複数のタグを指定する場合は withTags() を利用してください。
    # 
    # @todo Implement test case
    # @param {str} tag
    # @return {saklient.cloud.models.model_disk.Model_Disk}
    def with_tag(self, tag):
        Util.validate_type(tag, "str")
        return self._with_tag(tag)
    
    ## 指定したすべてのタグを持つリソースに絞り込みます。
    # 
    # @todo Implement test case
    # @param {str[]} tags
    # @return {saklient.cloud.models.model_disk.Model_Disk}
    def with_tags(self, tags):
        Util.validate_type(tags, "list")
        return self._with_tags(tags)
    
    ## 指定したDNFに合致するタグを持つリソースに絞り込みます。
    # 
    # @todo Implement test case
    # @param {str[][]} dnf
    # @return {saklient.cloud.models.model_disk.Model_Disk}
    def with_tag_dnf(self, dnf):
        Util.validate_type(dnf, "list")
        return self._with_tag_dnf(dnf)
    
    ## 名前でソートします。
    # 
    # @todo Implement test case
    # @param {bool} reverse=False
    # @return {saklient.cloud.models.model_disk.Model_Disk}
    def sort_by_name(self, reverse=False):
        Util.validate_type(reverse, "bool")
        return self._sort_by_name(reverse)
    
    ## @ignore
    # @param {saklient.cloud.client.Client} client
    def __init__(self, client):
        super(Model_Disk, self).__init__(client)
        Util.validate_type(client, "saklient.cloud.client.Client")
    
    ## 指定したサイズのディスクに絞り込みます。
    # 
    # @param {int} sizeGib
    # @return {saklient.cloud.models.model_disk.Model_Disk}
    def with_size_gib(self, sizeGib):
        Util.validate_type(sizeGib, "int")
        self._filter_by("SizeMB", [sizeGib * 1024])
        return self
    
    ## 指定したサーバへ接続されているディスクに絞り込みます。
    # 
    # @param {str} id
    # @return {saklient.cloud.models.model_disk.Model_Disk}
    def with_server_id(self, id):
        Util.validate_type(id, "str")
        self._filter_by("Server.ID", [id])
        return self
    
    ## サイズでソートします。
    # 
    # @param {bool} reverse=False
    # @return {saklient.cloud.models.model_disk.Model_Disk}
    def sort_by_size(self, reverse=False):
        Util.validate_type(reverse, "bool")
        self._sort("SizeMB", reverse)
        return self
    



# -*- coding:utf-8 -*-

from ...util import Util
from ..client import Client
from .resource import Resource
from .isoimage import IsoImage
from ..enums.eserverinstancestatus import EServerInstanceStatus

# module saklient.cloud.resources.serverinstance

class ServerInstance(Resource):
    ## サーバインスタンスの実体1つに対応し、属性の取得や操作を行うためのクラス。
    
    # (instance field) m_status
    
    # (instance field) m_before_status
    
    # (instance field) m_status_changed_at
    
    # (instance field) m_iso_image
    
    ## @ignore
    # @param {saklient.cloud.client.Client} client
    # @param {any} obj
    # @param {bool} wrapped=False
    def __init__(self, client, obj, wrapped=False):
        super(ServerInstance, self).__init__(client)
        Util.validate_type(client, "saklient.cloud.client.Client")
        Util.validate_type(wrapped, "bool")
        self.api_deserialize(obj, wrapped)
    
    ## サーバが起動しているときtrueを返します。
    # 
    # @return {bool}
    def is_up(self):
        return self.get_status() is not None and EServerInstanceStatus.compare(self.get_status(), EServerInstanceStatus.up) == 0
    
    ## サーバが停止しているときtrueを返します。
    # 
    # @return {bool}
    def is_down(self):
        return self.get_status() is None or EServerInstanceStatus.compare(self.get_status(), EServerInstanceStatus.down) == 0
    
    # (instance field) n_status = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_status(self):
        return self.m_status
    
    ## 起動状態 {@link EServerInstanceStatus}
    status = property(get_status, None, None)
    
    # (instance field) n_before_status = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_before_status(self):
        return self.m_before_status
    
    ## 前回の起動状態 {@link EServerInstanceStatus}
    before_status = property(get_before_status, None, None)
    
    # (instance field) n_status_changed_at = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {NativeDate}
    def get_status_changed_at(self):
        return self.m_status_changed_at
    
    ## 現在の起動状態に変化した日時
    status_changed_at = property(get_status_changed_at, None, None)
    
    # (instance field) n_iso_image = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {saklient.cloud.resources.isoimage.IsoImage}
    def get_iso_image(self):
        return self.m_iso_image
    
    ## 挿入されているISOイメージ
    iso_image = property(get_iso_image, None, None)
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {any} r
    def api_deserialize_impl(self, r):
        self.is_new = r is None
        if self.is_new:
            r = {
                
            }
        self.is_incomplete = False
        if Util.exists_path(r, "Status"):
            self.m_status = None if Util.get_by_path(r, "Status") is None else str(Util.get_by_path(r, "Status"))
        else:
            self.m_status = None
            self.is_incomplete = True
        self.n_status = False
        if Util.exists_path(r, "BeforeStatus"):
            self.m_before_status = None if Util.get_by_path(r, "BeforeStatus") is None else str(Util.get_by_path(r, "BeforeStatus"))
        else:
            self.m_before_status = None
            self.is_incomplete = True
        self.n_before_status = False
        if Util.exists_path(r, "StatusChangedAt"):
            self.m_status_changed_at = None if Util.get_by_path(r, "StatusChangedAt") is None else Util.str2date(str(Util.get_by_path(r, "StatusChangedAt")))
        else:
            self.m_status_changed_at = None
            self.is_incomplete = True
        self.n_status_changed_at = False
        if Util.exists_path(r, "CDROM"):
            self.m_iso_image = None if Util.get_by_path(r, "CDROM") is None else IsoImage(self._client, Util.get_by_path(r, "CDROM"))
        else:
            self.m_iso_image = None
            self.is_incomplete = True
        self.n_iso_image = False
    
    ## @ignore
    # @param {bool} withClean=False
    # @return {any}
    def api_serialize_impl(self, withClean=False):
        Util.validate_type(withClean, "bool")
        ret = {
            
        }
        if withClean or self.n_status:
            Util.set_by_path(ret, "Status", self.m_status)
        if withClean or self.n_before_status:
            Util.set_by_path(ret, "BeforeStatus", self.m_before_status)
        if withClean or self.n_status_changed_at:
            Util.set_by_path(ret, "StatusChangedAt", None if self.m_status_changed_at is None else Util.date2str(self.m_status_changed_at))
        if withClean or self.n_iso_image:
            Util.set_by_path(ret, "CDROM", (None if self.m_iso_image is None else self.m_iso_image.api_serialize(withClean)) if withClean else ({
                'ID': "0"
            } if self.m_iso_image is None else self.m_iso_image.api_serialize_id()))
        return ret
    



# -*- coding:utf-8 -*-

from ...errors.httpconflictexception import HttpConflictException

# module saklient.cloud.errors.limitcountinnetworkexception

class LimitCountInNetworkException(HttpConflictException):
    ## 要求を受け付けできません。ネットワーク内リソース数上限により、リソースの割り当てに失敗しました。
    
    ## @param {int} status
    # @param {str} code=None
    # @param {str} message=""
    def __init__(self, status, code=None, message=""):
        super(LimitCountInNetworkException, self).__init__(status, code, "要求を受け付けできません。ネットワーク内リソース数上限により、リソースの割り当てに失敗しました。" if message is None or message == "" else message)
    



# -*- coding:utf-8 -*-

from ...errors.saklientexception import SaklientException
from ..client import Client
from .resource import Resource
from .region import Region
from ...util import Util

# module saklient.cloud.resources.bridge

class Bridge(Resource):
    ## ブリッジの実体1つに対応し、属性の取得や操作を行うためのクラス。
    
    # (instance field) m_id
    
    # (instance field) m_name
    
    # (instance field) m_description
    
    # (instance field) m_region
    
    ## @private
    # @return {str}
    def _api_path(self):
        return "/bridge"
    
    ## @private
    # @return {str}
    def _root_key(self):
        return "Bridge"
    
    ## @private
    # @return {str}
    def _root_key_m(self):
        return "Bridges"
    
    ## @private
    # @return {str}
    def _class_name(self):
        return "Bridge"
    
    ## @private
    # @return {str}
    def _id(self):
        return self.get_id()
    
    ## このローカルオブジェクトに現在設定されているリソース情報をAPIに送信し、新規作成または上書き保存します。
    # 
    # @return {saklient.cloud.resources.bridge.Bridge} this
    def save(self):
        return self._save()
    
    ## 最新のリソース情報を再取得します。
    # 
    # @return {saklient.cloud.resources.bridge.Bridge} this
    def reload(self):
        return self._reload()
    
    ## @ignore
    # @param {saklient.cloud.client.Client} client
    # @param {any} obj
    # @param {bool} wrapped=False
    def __init__(self, client, obj, wrapped=False):
        super(Bridge, self).__init__(client)
        Util.validate_type(client, "saklient.cloud.client.Client")
        Util.validate_type(wrapped, "bool")
        self.api_deserialize(obj, wrapped)
    
    # (instance field) n_id = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_id(self):
        return self.m_id
    
    ## ID
    id = property(get_id, None, None)
    
    # (instance field) n_name = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_name(self):
        return self.m_name
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {str} v
    # @return {str}
    def set_name(self, v):
        Util.validate_type(v, "str")
        self.m_name = v
        self.n_name = True
        return self.m_name
    
    ## 名前
    name = property(get_name, set_name, None)
    
    # (instance field) n_description = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_description(self):
        return self.m_description
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {str} v
    # @return {str}
    def set_description(self, v):
        Util.validate_type(v, "str")
        self.m_description = v
        self.n_description = True
        return self.m_description
    
    ## 説明
    description = property(get_description, set_description, None)
    
    # (instance field) n_region = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {saklient.cloud.resources.region.Region}
    def get_region(self):
        return self.m_region
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {saklient.cloud.resources.region.Region} v
    # @return {saklient.cloud.resources.region.Region}
    def set_region(self, v):
        Util.validate_type(v, "saklient.cloud.resources.region.Region")
        if not self.is_new:
            raise SaklientException("immutable_field", "Immutable fields cannot be modified after the resource creation: " + "saklient.cloud.resources.bridge.Bridge#region")
        self.m_region = v
        self.n_region = True
        return self.m_region
    
    ## リージョン
    region = property(get_region, set_region, None)
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {any} r
    def api_deserialize_impl(self, r):
        self.is_new = r is None
        if self.is_new:
            r = {
                
            }
        self.is_incomplete = False
        if Util.exists_path(r, "ID"):
            self.m_id = None if Util.get_by_path(r, "ID") is None else str(Util.get_by_path(r, "ID"))
        else:
            self.m_id = None
            self.is_incomplete = True
        self.n_id = False
        if Util.exists_path(r, "Name"):
            self.m_name = None if Util.get_by_path(r, "Name") is None else str(Util.get_by_path(r, "Name"))
        else:
            self.m_name = None
            self.is_incomplete = True
        self.n_name = False
        if Util.exists_path(r, "Description"):
            self.m_description = None if Util.get_by_path(r, "Description") is None else str(Util.get_by_path(r, "Description"))
        else:
            self.m_description = None
            self.is_incomplete = True
        self.n_description = False
        if Util.exists_path(r, "Region"):
            self.m_region = None if Util.get_by_path(r, "Region") is None else Region(self._client, Util.get_by_path(r, "Region"))
        else:
            self.m_region = None
            self.is_incomplete = True
        self.n_region = False
    
    ## @ignore
    # @param {bool} withClean=False
    # @return {any}
    def api_serialize_impl(self, withClean=False):
        Util.validate_type(withClean, "bool")
        missing = []
        ret = {
            
        }
        if withClean or self.n_id:
            Util.set_by_path(ret, "ID", self.m_id)
        if withClean or self.n_name:
            Util.set_by_path(ret, "Name", self.m_name)
        else:
            if self.is_new:
                missing.append("name")
        if withClean or self.n_description:
            Util.set_by_path(ret, "Description", self.m_description)
        if withClean or self.n_region:
            Util.set_by_path(ret, "Region", (None if self.m_region is None else self.m_region.api_serialize(withClean)) if withClean else ({
                'ID': "0"
            } if self.m_region is None else self.m_region.api_serialize_id()))
        else:
            if self.is_new:
                missing.append("region")
        if len(missing) > 0:
            raise SaklientException("required_field", "Required fields must be set before the Bridge creation: " + ", ".join(missing))
        return ret
    



# -*- coding:utf-8 -*-

from .models.model_region import Model_Region
from .client import Client
from ..util import Util

# module saklient.cloud.facility

class Facility:
    ## 設備情報にアクセスするためのモデルを集めたクラス。
    
    # (instance field) _region
    
    ## @return {saklient.cloud.models.model_region.Model_Region}
    def get_region(self):
        return self._region
    
    ## リージョン情報。
    region = property(get_region, None, None)
    
    ## @ignore
    # @param {saklient.cloud.client.Client} client
    def __init__(self, client):
        Util.validate_type(client, "saklient.cloud.client.Client")
        self._region = Model_Region(client)
    



from distutils.core import setup
setup(
    name = 'saklient',
    version = '0.0.2.5',
    description = 'SAKURA Internet API Client Library',
    author='SAKURA Internet Inc.',
    author_email='dev-support-ml@sakura.ad.jp',
    url = 'http://cloud.sakura.ad.jp/',
    keywords = ['cloud'],
    packages = [
        'saklient',
        'saklient.cloud',
        'saklient.cloud.enums',
        'saklient.cloud.errors',
        'saklient.cloud.models',
        'saklient.cloud.resources',
        'saklient.errors'
    ],
    classifiers = [
        'Programming Language :: Python :: 3',
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)



# -*- coding:utf-8 -*-

from ...util import Util
from .lbserver import LbServer

# module saklient.cloud.resources.lbvirtualip

class LbVirtualIp:
    ## ロードバランサの仮想IPアドレス。
    
    # (instance field) _virtual_ip_address
    
    ## @return {str}
    def get_virtual_ip_address(self):
        return self._virtual_ip_address
    
    ## @param {str} v
    # @return {str}
    def set_virtual_ip_address(self, v):
        Util.validate_type(v, "str")
        self._virtual_ip_address = v
        return self._virtual_ip_address
    
    ## VIPアドレス
    virtual_ip_address = property(get_virtual_ip_address, set_virtual_ip_address, None)
    
    # (instance field) _port
    
    ## @return {int}
    def get_port(self):
        return self._port
    
    ## @param {int} v
    # @return {int}
    def set_port(self, v):
        Util.validate_type(v, "int")
        self._port = v
        return self._port
    
    ## ポート番号
    port = property(get_port, set_port, None)
    
    # (instance field) _delay_loop
    
    ## @return {int}
    def get_delay_loop(self):
        return self._delay_loop
    
    ## @param {int} v
    # @return {int}
    def set_delay_loop(self, v):
        Util.validate_type(v, "int")
        self._delay_loop = v
        return self._delay_loop
    
    ## チェック間隔 [秒]
    delay_loop = property(get_delay_loop, set_delay_loop, None)
    
    # (instance field) _servers
    
    ## @return {saklient.cloud.resources.lbserver.LbServer[]}
    def get_servers(self):
        return self._servers
    
    ## 実サーバ {@link LbServer} の配列
    servers = property(get_servers, None, None)
    
    ## @ignore
    # @param {any} obj=None
    def __init__(self, obj=None):
        if obj is None:
            obj = {}
        vip = Util.get_by_path_any([obj], ["VirtualIPAddress", "virtualIpAddress", "virtual_ip_address", "vip"])
        self._virtual_ip_address = vip
        port = Util.get_by_path_any([obj], ["Port", "port"])
        self._port = None
        if port is not None:
            self._port = int(port)
        if self._port == 0:
            self._port = None
        delayLoop = Util.get_by_path_any([obj], ["DelayLoop", "delayLoop", "delay_loop", "delay"])
        self._delay_loop = None
        if delayLoop is not None:
            self._delay_loop = int(delayLoop)
        if self._delay_loop == 0:
            self._delay_loop = None
        self._servers = []
        serversDyn = Util.get_by_path_any([obj], ["Servers", "servers"])
        if serversDyn is not None:
            servers = serversDyn
            for server in servers:
                self._servers.append(LbServer(server))
    
    ## @param {any} settings=None
    # @return {saklient.cloud.resources.lbserver.LbServer}
    def add_server(self, settings=None):
        ret = LbServer(settings)
        self._servers.append(ret)
        return ret
    
    ## @return {any}
    def to_raw_settings(self):
        servers = []
        for server in self._servers:
            servers.append(server.to_raw_settings())
        return {
            'VirtualIPAddress': self._virtual_ip_address,
            'Port': self._port,
            'DelayLoop': self._delay_loop,
            'Servers': servers
        }
    
    ## @param {str} address
    # @return {saklient.cloud.resources.lbserver.LbServer}
    def get_server_by_address(self, address):
        Util.validate_type(address, "str")
        for srv in self._servers:
            if srv.ip_address == address:
                return srv
        return None
    
    ## @param {any[]} srvs
    # @return {saklient.cloud.resources.lbvirtualip.LbVirtualIp}
    def update_status(self, srvs):
        Util.validate_type(srvs, "list")
        for srvDyn in srvs:
            ip = (srvDyn["IPAddress"] if "IPAddress" in srvDyn else None)
            srv = self.get_server_by_address(ip)
            if srv is None:
                next
            srv.update_status(srvDyn)
        return self
    



# -*- coding:utf-8 -*-

from .httpexception import HttpException
from .httpbadgatewayexception import HttpBadGatewayException
from .httpbadrequestexception import HttpBadRequestException
from .httpconflictexception import HttpConflictException
from .httpexpectationfailedexception import HttpExpectationFailedException
from .httpfaileddependencyexception import HttpFailedDependencyException
from .httpforbiddenexception import HttpForbiddenException
from .httpgatewaytimeoutexception import HttpGatewayTimeoutException
from .httpgoneexception import HttpGoneException
from .httphttpversionnotsupportedexception import HttpHttpVersionNotSupportedException
from .httpinsufficientstorageexception import HttpInsufficientStorageException
from .httpinternalservererrorexception import HttpInternalServerErrorException
from .httplengthrequiredexception import HttpLengthRequiredException
from .httplockedexception import HttpLockedException
from .httpmethodnotallowedexception import HttpMethodNotAllowedException
from .httpnotacceptableexception import HttpNotAcceptableException
from .httpnotextendedexception import HttpNotExtendedException
from .httpnotfoundexception import HttpNotFoundException
from .httpnotimplementedexception import HttpNotImplementedException
from .httppaymentrequiredexception import HttpPaymentRequiredException
from .httppreconditionfailedexception import HttpPreconditionFailedException
from .httpproxyauthenticationrequiredexception import HttpProxyAuthenticationRequiredException
from .httprequestentitytoolargeexception import HttpRequestEntityTooLargeException
from .httprequesttimeoutexception import HttpRequestTimeoutException
from .httprequesturitoolongexception import HttpRequestUriTooLongException
from .httprequestedrangenotsatisfiableexception import HttpRequestedRangeNotSatisfiableException
from .httpserviceunavailableexception import HttpServiceUnavailableException
from .httpunauthorizedexception import HttpUnauthorizedException
from .httpunprocessableentityexception import HttpUnprocessableEntityException
from .httpunsupportedmediatypeexception import HttpUnsupportedMediaTypeException
from .httpupgraderequiredexception import HttpUpgradeRequiredException
from .httpvariantalsonegotiatesexception import HttpVariantAlsoNegotiatesException
from ..cloud.errors.accessapikeydisabledexception import AccessApiKeyDisabledException
from ..cloud.errors.accesssakuraexception import AccessSakuraException
from ..cloud.errors.accessstaffexception import AccessStaffException
from ..cloud.errors.accesstokenexception import AccessTokenException
from ..cloud.errors.accessxhrorapikeyexception import AccessXhrOrApiKeyException
from ..cloud.errors.accountnotfoundexception import AccountNotFoundException
from ..cloud.errors.accountnotspecifiedexception import AccountNotSpecifiedException
from ..cloud.errors.ambiguousidentifierexception import AmbiguousIdentifierException
from ..cloud.errors.ambiguouszoneexception import AmbiguousZoneException
from ..cloud.errors.apiproxytimeoutexception import ApiProxyTimeoutException
from ..cloud.errors.apiproxytimeoutnongetexception import ApiProxyTimeoutNonGetException
from ..cloud.errors.archiveisincompleteexception import ArchiveIsIncompleteException
from ..cloud.errors.bootfailurebylockexception import BootFailureByLockException
from ..cloud.errors.bootfailureingroupexception import BootFailureInGroupException
from ..cloud.errors.busyexception import BusyException
from ..cloud.errors.cantresizesmallerexception import CantResizeSmallerException
from ..cloud.errors.cdromdevicelockedexception import CdromDeviceLockedException
from ..cloud.errors.cdromdisabledexception import CdromDisabledException
from ..cloud.errors.cdrominuseexception import CdromInUseException
from ..cloud.errors.cdromisincompleteexception import CdromIsIncompleteException
from ..cloud.errors.connecttosameswitchexception import ConnectToSameSwitchException
from ..cloud.errors.contractcreationexception import ContractCreationException
from ..cloud.errors.copytoitselfexception import CopyToItselfException
from ..cloud.errors.deletediskb4templateexception import DeleteDiskB4TemplateException
from ..cloud.errors.deleteipv6netsfirstexception import DeleteIpV6NetsFirstException
from ..cloud.errors.deleteresb4accountexception import DeleteResB4AccountException
from ..cloud.errors.deleterouterb4switchexception import DeleteRouterB4SwitchException
from ..cloud.errors.deletestaticroutefirstexception import DeleteStaticRouteFirstException
from ..cloud.errors.disabledinsandboxexception import DisabledInSandboxException
from ..cloud.errors.disconnectb4deleteexception import DisconnectB4DeleteException
from ..cloud.errors.disconnectb4updateexception import DisconnectB4UpdateException
from ..cloud.errors.diskconnectionlimitexception import DiskConnectionLimitException
from ..cloud.errors.diskiscopyingexception import DiskIsCopyingException
from ..cloud.errors.diskisnotavailableexception import DiskIsNotAvailableException
from ..cloud.errors.disklicensemismatchexception import DiskLicenseMismatchException
from ..cloud.errors.diskorssinmigrationexception import DiskOrSsInMigrationException
from ..cloud.errors.diskstockrunoutexception import DiskStockRunOutException
from ..cloud.errors.dnsarecordnotfoundexception import DnsARecordNotFoundException
from ..cloud.errors.dnsaaaarecordnotfoundexception import DnsAaaaRecordNotFoundException
from ..cloud.errors.dnsptrupdatefailureexception import DnsPtrUpdateFailureException
from ..cloud.errors.dontcreateinsandboxexception import DontCreateInSandboxException
from ..cloud.errors.duplicateaccountcodeexception import DuplicateAccountCodeException
from ..cloud.errors.duplicateentryexception import DuplicateEntryException
from ..cloud.errors.duplicateusercodeexception import DuplicateUserCodeException
from ..cloud.errors.filenotuploadedexception import FileNotUploadedException
from ..cloud.errors.filterarraycomparisonexception import FilterArrayComparisonException
from ..cloud.errors.filterbadoperatorexception import FilterBadOperatorException
from ..cloud.errors.filternullcomparisonexception import FilterNullComparisonException
from ..cloud.errors.filterunknownoperatorexception import FilterUnknownOperatorException
from ..cloud.errors.ftpcannotcloseexception import FtpCannotCloseException
from ..cloud.errors.ftpisalreadycloseexception import FtpIsAlreadyCloseException
from ..cloud.errors.ftpisalreadyopenexception import FtpIsAlreadyOpenException
from ..cloud.errors.ftpmustbeclosedexception import FtpMustBeClosedException
from ..cloud.errors.hostoperationfailureexception import HostOperationFailureException
from ..cloud.errors.illegaldasusageexception import IllegalDasUsageException
from ..cloud.errors.inmigrationexception import InMigrationException
from ..cloud.errors.invalidformatexception import InvalidFormatException
from ..cloud.errors.invalidparamcombexception import InvalidParamCombException
from ..cloud.errors.invalidrangeexception import InvalidRangeException
from ..cloud.errors.invaliduriargumentexception import InvalidUriArgumentException
from ..cloud.errors.ipv6netalreadyattachedexception import IpV6NetAlreadyAttachedException
from ..cloud.errors.limitcountinaccountexception import LimitCountInAccountException
from ..cloud.errors.limitcountinmemberexception import LimitCountInMemberException
from ..cloud.errors.limitcountinnetworkexception import LimitCountInNetworkException
from ..cloud.errors.limitcountinrouterexception import LimitCountInRouterException
from ..cloud.errors.limitcountinzoneexception import LimitCountInZoneException
from ..cloud.errors.limitmemoryinaccountexception import LimitMemoryInAccountException
from ..cloud.errors.limitsizeinaccountexception import LimitSizeInAccountException
from ..cloud.errors.missingisoimageexception import MissingIsoImageException
from ..cloud.errors.missingparamexception import MissingParamException
from ..cloud.errors.mustbeofsamezoneexception import MustBeOfSameZoneException
from ..cloud.errors.nodisplayresponseexception import NoDisplayResponseException
from ..cloud.errors.notforrouterexception import NotForRouterException
from ..cloud.errors.notreplicatingexception import NotReplicatingException
from ..cloud.errors.notwithhybridconnexception import NotWithHybridconnException
from ..cloud.errors.oldstorageplanexception import OldStoragePlanException
from ..cloud.errors.operationfailureexception import OperationFailureException
from ..cloud.errors.operationtimeoutexception import OperationTimeoutException
from ..cloud.errors.originalhashmismatchexception import OriginalHashMismatchException
from ..cloud.errors.packetfilterapplyingexception import PacketFilterApplyingException
from ..cloud.errors.packetfilterversionmismatchexception import PacketFilterVersionMismatchException
from ..cloud.errors.paramipnotfoundexception import ParamIpNotFoundException
from ..cloud.errors.paramresnotfoundexception import ParamResNotFoundException
from ..cloud.errors.paymentcreditcardexception import PaymentCreditCardException
from ..cloud.errors.paymentpaymentexception import PaymentPaymentException
from ..cloud.errors.paymentregistrationexception import PaymentRegistrationException
from ..cloud.errors.paymenttelcertificationexception import PaymentTelCertificationException
from ..cloud.errors.paymentunpayableexception import PaymentUnpayableException
from ..cloud.errors.penaltyoperationexception import PenaltyOperationException
from ..cloud.errors.replicaalreadyexistsexception import ReplicaAlreadyExistsException
from ..cloud.errors.replicanotfoundexception import ReplicaNotFoundException
from ..cloud.errors.resalreadyconnectedexception import ResAlreadyConnectedException
from ..cloud.errors.resalreadydisconnectedexception import ResAlreadyDisconnectedException
from ..cloud.errors.resalreadyexistsexception import ResAlreadyExistsException
from ..cloud.errors.resusedinzoneexception import ResUsedInZoneException
from ..cloud.errors.resourcepathnotfoundexception import ResourcePathNotFoundException
from ..cloud.errors.runoutofipaddressexception import RunOutOfIpAddressException
from ..cloud.errors.samelicenserequiredexception import SameLicenseRequiredException
from ..cloud.errors.servercouldnotstopexception import ServerCouldNotStopException
from ..cloud.errors.serveriscleaningexception import ServerIsCleaningException
from ..cloud.errors.serveroperationfailureexception import ServerOperationFailureException
from ..cloud.errors.serverpowermustbedownexception import ServerPowerMustBeDownException
from ..cloud.errors.serverpowermustbeupexception import ServerPowerMustBeUpException
from ..cloud.errors.servicetemporarilyunavailableexception import ServiceTemporarilyUnavailableException
from ..cloud.errors.sizemismatchexception import SizeMismatchException
from ..cloud.errors.snapshotinmigrationexception import SnapshotInMigrationException
from ..cloud.errors.stillcreatingexception import StillCreatingException
from ..cloud.errors.storageabnormalexception import StorageAbnormalException
from ..cloud.errors.storageoperationfailureexception import StorageOperationFailureException
from ..cloud.errors.switchhybridconnectedexception import SwitchHybridConnectedException
from ..cloud.errors.templateftpisopenexception import TemplateFtpIsOpenException
from ..cloud.errors.templateisincompleteexception import TemplateIsIncompleteException
from ..cloud.errors.toomanyrequestexception import TooManyRequestException
from ..cloud.errors.unknownexception import UnknownException
from ..cloud.errors.unknownostypeexception import UnknownOsTypeException
from ..cloud.errors.unsupportedresclassexception import UnsupportedResClassException
from ..cloud.errors.usernotspecifiedexception import UserNotSpecifiedException
from ..cloud.errors.vncproxyrequestfailureexception import VncProxyRequestFailureException
from ..util import Util

# module saklient.errors.exceptionfactory

class ExceptionFactory:
    
    ## @static
    # @param {int} status
    # @param {str} code=None
    # @param {str} message=""
    # @return {saklient.errors.httpexception.HttpException}
    @staticmethod
    def create(status, code=None, message=""):
        if code == "access_apikey_disabled":
            return AccessApiKeyDisabledException(status, code, message)
        elif code == "access_sakura":
            return AccessSakuraException(status, code, message)
        elif code == "access_staff":
            return AccessStaffException(status, code, message)
        elif code == "access_token":
            return AccessTokenException(status, code, message)
        elif code == "access_xhr_or_apikey":
            return AccessXhrOrApiKeyException(status, code, message)
        elif code == "account_not_found":
            return AccountNotFoundException(status, code, message)
        elif code == "account_not_specified":
            return AccountNotSpecifiedException(status, code, message)
        elif code == "ambiguous_identifier":
            return AmbiguousIdentifierException(status, code, message)
        elif code == "ambiguous_zone":
            return AmbiguousZoneException(status, code, message)
        elif code == "apiproxy_timeout":
            return ApiProxyTimeoutException(status, code, message)
        elif code == "apiproxy_timeout_non_get":
            return ApiProxyTimeoutNonGetException(status, code, message)
        elif code == "archive_is_incomplete":
            return ArchiveIsIncompleteException(status, code, message)
        elif code == "bad_gateway":
            return HttpBadGatewayException(status, code, message)
        elif code == "bad_request":
            return HttpBadRequestException(status, code, message)
        elif code == "boot_failure_by_lock":
            return BootFailureByLockException(status, code, message)
        elif code == "boot_failure_in_group":
            return BootFailureInGroupException(status, code, message)
        elif code == "busy":
            return BusyException(status, code, message)
        elif code == "cant_resize_smaller":
            return CantResizeSmallerException(status, code, message)
        elif code == "cdrom_device_locked":
            return CdromDeviceLockedException(status, code, message)
        elif code == "cdrom_disabled":
            return CdromDisabledException(status, code, message)
        elif code == "cdrom_in_use":
            return CdromInUseException(status, code, message)
        elif code == "cdrom_is_incomplete":
            return CdromIsIncompleteException(status, code, message)
        elif code == "conflict":
            return HttpConflictException(status, code, message)
        elif code == "connect_to_same_switch":
            return ConnectToSameSwitchException(status, code, message)
        elif code == "contract_creation":
            return ContractCreationException(status, code, message)
        elif code == "copy_to_itself":
            return CopyToItselfException(status, code, message)
        elif code == "delete_disk_b4_template":
            return DeleteDiskB4TemplateException(status, code, message)
        elif code == "delete_ipv6nets_first":
            return DeleteIpV6NetsFirstException(status, code, message)
        elif code == "delete_res_b4_account":
            return DeleteResB4AccountException(status, code, message)
        elif code == "delete_router_b4_switch":
            return DeleteRouterB4SwitchException(status, code, message)
        elif code == "delete_static_route_first":
            return DeleteStaticRouteFirstException(status, code, message)
        elif code == "disabled_in_sandbox":
            return DisabledInSandboxException(status, code, message)
        elif code == "disconnect_b4_delete":
            return DisconnectB4DeleteException(status, code, message)
        elif code == "disconnect_b4_update":
            return DisconnectB4UpdateException(status, code, message)
        elif code == "disk_connection_limit":
            return DiskConnectionLimitException(status, code, message)
        elif code == "disk_is_copying":
            return DiskIsCopyingException(status, code, message)
        elif code == "disk_is_not_available":
            return DiskIsNotAvailableException(status, code, message)
        elif code == "disk_license_mismatch":
            return DiskLicenseMismatchException(status, code, message)
        elif code == "disk_stock_run_out":
            return DiskStockRunOutException(status, code, message)
        elif code == "diskorss_in_migration":
            return DiskOrSsInMigrationException(status, code, message)
        elif code == "dns_a_record_not_found":
            return DnsARecordNotFoundException(status, code, message)
        elif code == "dns_aaaa_record_not_found":
            return DnsAaaaRecordNotFoundException(status, code, message)
        elif code == "dns_ptr_update_failure":
            return DnsPtrUpdateFailureException(status, code, message)
        elif code == "dont_create_in_sandbox":
            return DontCreateInSandboxException(status, code, message)
        elif code == "duplicate_account_code":
            return DuplicateAccountCodeException(status, code, message)
        elif code == "duplicate_entry":
            return DuplicateEntryException(status, code, message)
        elif code == "duplicate_user_code":
            return DuplicateUserCodeException(status, code, message)
        elif code == "expectation_failed":
            return HttpExpectationFailedException(status, code, message)
        elif code == "failed_dependency":
            return HttpFailedDependencyException(status, code, message)
        elif code == "file_not_uploaded":
            return FileNotUploadedException(status, code, message)
        elif code == "filter_array_comparison":
            return FilterArrayComparisonException(status, code, message)
        elif code == "filter_bad_operator":
            return FilterBadOperatorException(status, code, message)
        elif code == "filter_null_comparison":
            return FilterNullComparisonException(status, code, message)
        elif code == "filter_unknown_operator":
            return FilterUnknownOperatorException(status, code, message)
        elif code == "forbidden":
            return HttpForbiddenException(status, code, message)
        elif code == "ftp_cannot_close":
            return FtpCannotCloseException(status, code, message)
        elif code == "ftp_is_already_close":
            return FtpIsAlreadyCloseException(status, code, message)
        elif code == "ftp_is_already_open":
            return FtpIsAlreadyOpenException(status, code, message)
        elif code == "ftp_must_be_closed":
            return FtpMustBeClosedException(status, code, message)
        elif code == "gateway_timeout":
            return HttpGatewayTimeoutException(status, code, message)
        elif code == "gone":
            return HttpGoneException(status, code, message)
        elif code == "host_operation_failure":
            return HostOperationFailureException(status, code, message)
        elif code == "http_version_not_supported":
            return HttpHttpVersionNotSupportedException(status, code, message)
        elif code == "illegal_das_usage":
            return IllegalDasUsageException(status, code, message)
        elif code == "in_migration":
            return InMigrationException(status, code, message)
        elif code == "insufficient_storage":
            return HttpInsufficientStorageException(status, code, message)
        elif code == "internal_server_error":
            return HttpInternalServerErrorException(status, code, message)
        elif code == "invalid_format":
            return InvalidFormatException(status, code, message)
        elif code == "invalid_param_comb":
            return InvalidParamCombException(status, code, message)
        elif code == "invalid_range":
            return InvalidRangeException(status, code, message)
        elif code == "invalid_uri_argument":
            return InvalidUriArgumentException(status, code, message)
        elif code == "ipv6net_already_attached":
            return IpV6NetAlreadyAttachedException(status, code, message)
        elif code == "length_required":
            return HttpLengthRequiredException(status, code, message)
        elif code == "limit_count_in_account":
            return LimitCountInAccountException(status, code, message)
        elif code == "limit_count_in_member":
            return LimitCountInMemberException(status, code, message)
        elif code == "limit_count_in_network":
            return LimitCountInNetworkException(status, code, message)
        elif code == "limit_count_in_router":
            return LimitCountInRouterException(status, code, message)
        elif code == "limit_count_in_zone":
            return LimitCountInZoneException(status, code, message)
        elif code == "limit_memory_in_account":
            return LimitMemoryInAccountException(status, code, message)
        elif code == "limit_size_in_account":
            return LimitSizeInAccountException(status, code, message)
        elif code == "locked":
            return HttpLockedException(status, code, message)
        elif code == "method_not_allowed":
            return HttpMethodNotAllowedException(status, code, message)
        elif code == "missing_iso_image":
            return MissingIsoImageException(status, code, message)
        elif code == "missing_param":
            return MissingParamException(status, code, message)
        elif code == "must_be_of_same_zone":
            return MustBeOfSameZoneException(status, code, message)
        elif code == "no_display_response":
            return NoDisplayResponseException(status, code, message)
        elif code == "not_acceptable":
            return HttpNotAcceptableException(status, code, message)
        elif code == "not_extended":
            return HttpNotExtendedException(status, code, message)
        elif code == "not_for_router":
            return NotForRouterException(status, code, message)
        elif code == "not_found":
            return HttpNotFoundException(status, code, message)
        elif code == "not_implemented":
            return HttpNotImplementedException(status, code, message)
        elif code == "not_replicating":
            return NotReplicatingException(status, code, message)
        elif code == "not_with_hybridconn":
            return NotWithHybridconnException(status, code, message)
        elif code == "old_storage_plan":
            return OldStoragePlanException(status, code, message)
        elif code == "operation_failure":
            return OperationFailureException(status, code, message)
        elif code == "operation_timeout":
            return OperationTimeoutException(status, code, message)
        elif code == "original_hash_mismatch":
            return OriginalHashMismatchException(status, code, message)
        elif code == "packetfilter_applying":
            return PacketFilterApplyingException(status, code, message)
        elif code == "packetfilter_version_mismatch":
            return PacketFilterVersionMismatchException(status, code, message)
        elif code == "param_ip_not_found":
            return ParamIpNotFoundException(status, code, message)
        elif code == "param_res_not_found":
            return ParamResNotFoundException(status, code, message)
        elif code == "payment_creditcard":
            return PaymentCreditCardException(status, code, message)
        elif code == "payment_payment":
            return PaymentPaymentException(status, code, message)
        elif code == "payment_registration":
            return PaymentRegistrationException(status, code, message)
        elif code == "payment_required":
            return HttpPaymentRequiredException(status, code, message)
        elif code == "payment_telcertification":
            return PaymentTelCertificationException(status, code, message)
        elif code == "payment_unpayable":
            return PaymentUnpayableException(status, code, message)
        elif code == "penalty_operation":
            return PenaltyOperationException(status, code, message)
        elif code == "precondition_failed":
            return HttpPreconditionFailedException(status, code, message)
        elif code == "proxy_authentication_required":
            return HttpProxyAuthenticationRequiredException(status, code, message)
        elif code == "replica_already_exists":
            return ReplicaAlreadyExistsException(status, code, message)
        elif code == "replica_not_found":
            return ReplicaNotFoundException(status, code, message)
        elif code == "request_entity_too_large":
            return HttpRequestEntityTooLargeException(status, code, message)
        elif code == "request_timeout":
            return HttpRequestTimeoutException(status, code, message)
        elif code == "request_uri_too_long":
            return HttpRequestUriTooLongException(status, code, message)
        elif code == "requested_range_not_satisfiable":
            return HttpRequestedRangeNotSatisfiableException(status, code, message)
        elif code == "res_already_connected":
            return ResAlreadyConnectedException(status, code, message)
        elif code == "res_already_disconnected":
            return ResAlreadyDisconnectedException(status, code, message)
        elif code == "res_already_exists":
            return ResAlreadyExistsException(status, code, message)
        elif code == "res_used_in_zone":
            return ResUsedInZoneException(status, code, message)
        elif code == "resource_path_not_found":
            return ResourcePathNotFoundException(status, code, message)
        elif code == "run_out_of_ipaddress":
            return RunOutOfIpAddressException(status, code, message)
        elif code == "same_license_required":
            return SameLicenseRequiredException(status, code, message)
        elif code == "server_could_not_stop":
            return ServerCouldNotStopException(status, code, message)
        elif code == "server_is_cleaning":
            return ServerIsCleaningException(status, code, message)
        elif code == "server_operation_failure":
            return ServerOperationFailureException(status, code, message)
        elif code == "server_power_must_be_down":
            return ServerPowerMustBeDownException(status, code, message)
        elif code == "server_power_must_be_up":
            return ServerPowerMustBeUpException(status, code, message)
        elif code == "service_temporarily_unavailable":
            return ServiceTemporarilyUnavailableException(status, code, message)
        elif code == "service_unavailable":
            return HttpServiceUnavailableException(status, code, message)
        elif code == "size_mismatch":
            return SizeMismatchException(status, code, message)
        elif code == "snapshot_in_migration":
            return SnapshotInMigrationException(status, code, message)
        elif code == "still_creating":
            return StillCreatingException(status, code, message)
        elif code == "storage_abnormal":
            return StorageAbnormalException(status, code, message)
        elif code == "storage_operation_failure":
            return StorageOperationFailureException(status, code, message)
        elif code == "switch_hybrid_connected":
            return SwitchHybridConnectedException(status, code, message)
        elif code == "template_ftp_is_open":
            return TemplateFtpIsOpenException(status, code, message)
        elif code == "template_is_incomplete":
            return TemplateIsIncompleteException(status, code, message)
        elif code == "too_many_request":
            return TooManyRequestException(status, code, message)
        elif code == "unauthorized":
            return HttpUnauthorizedException(status, code, message)
        elif code == "unknown":
            return UnknownException(status, code, message)
        elif code == "unknown_os_type":
            return UnknownOsTypeException(status, code, message)
        elif code == "unprocessable_entity":
            return HttpUnprocessableEntityException(status, code, message)
        elif code == "unsupported_media_type":
            return HttpUnsupportedMediaTypeException(status, code, message)
        elif code == "unsupported_res_class":
            return UnsupportedResClassException(status, code, message)
        elif code == "upgrade_required":
            return HttpUpgradeRequiredException(status, code, message)
        elif code == "user_not_specified":
            return UserNotSpecifiedException(status, code, message)
        elif code == "variant_also_negotiates":
            return HttpVariantAlsoNegotiatesException(status, code, message)
        elif code == "vnc_proxy_request_failure":
            return VncProxyRequestFailureException(status, code, message)
        if status == 400:
            return HttpBadRequestException(status, code, message)
        elif status == 401:
            return HttpUnauthorizedException(status, code, message)
        elif status == 402:
            return HttpPaymentRequiredException(status, code, message)
        elif status == 403:
            return HttpForbiddenException(status, code, message)
        elif status == 404:
            return HttpNotFoundException(status, code, message)
        elif status == 405:
            return HttpMethodNotAllowedException(status, code, message)
        elif status == 406:
            return HttpNotAcceptableException(status, code, message)
        elif status == 407:
            return HttpProxyAuthenticationRequiredException(status, code, message)
        elif status == 408:
            return HttpRequestTimeoutException(status, code, message)
        elif status == 409:
            return HttpConflictException(status, code, message)
        elif status == 410:
            return HttpGoneException(status, code, message)
        elif status == 411:
            return HttpLengthRequiredException(status, code, message)
        elif status == 412:
            return HttpPreconditionFailedException(status, code, message)
        elif status == 413:
            return HttpRequestEntityTooLargeException(status, code, message)
        elif status == 415:
            return HttpUnsupportedMediaTypeException(status, code, message)
        elif status == 416:
            return HttpRequestedRangeNotSatisfiableException(status, code, message)
        elif status == 417:
            return HttpExpectationFailedException(status, code, message)
        elif status == 422:
            return HttpUnprocessableEntityException(status, code, message)
        elif status == 423:
            return HttpLockedException(status, code, message)
        elif status == 424:
            return HttpFailedDependencyException(status, code, message)
        elif status == 426:
            return HttpUpgradeRequiredException(status, code, message)
        elif status == 500:
            return HttpRequestUriTooLongException(status, code, message)
        elif status == 501:
            return HttpNotImplementedException(status, code, message)
        elif status == 502:
            return HttpBadGatewayException(status, code, message)
        elif status == 503:
            return HttpServiceUnavailableException(status, code, message)
        elif status == 504:
            return HttpGatewayTimeoutException(status, code, message)
        elif status == 505:
            return HttpHttpVersionNotSupportedException(status, code, message)
        elif status == 506:
            return HttpVariantAlsoNegotiatesException(status, code, message)
        elif status == 507:
            return HttpInsufficientStorageException(status, code, message)
        elif status == 510:
            return HttpNotExtendedException(status, code, message)
        return HttpException(status, code, message)
    



# -*- coding:utf-8 -*-

from ...errors.saklientexception import SaklientException
from ..client import Client
from .resource import Resource
from .icon import Icon
from .iface import Iface
from .swytch import Swytch
from ..enums.eapplianceclass import EApplianceClass
from ..enums.eavailability import EAvailability
from ..enums.eserverinstancestatus import EServerInstanceStatus
from ...util import Util

# module saklient.cloud.resources.appliance

class Appliance(Resource):
    ## アプライアンスの実体1つに対応し、属性の取得や操作を行うためのクラス。
    
    # (instance field) m_id
    
    # (instance field) m_clazz
    
    # (instance field) m_name
    
    # (instance field) m_description
    
    # (instance field) m_tags
    
    # (instance field) m_icon
    
    # (instance field) m_plan_id
    
    # (instance field) m_ifaces
    
    # (instance field) m_raw_annotation
    
    # (instance field) m_raw_settings
    
    # (instance field) m_raw_settings_hash
    
    # (instance field) m_status
    
    # (instance field) m_service_class
    
    # (instance field) m_availability
    
    # (instance field) m_swytch_id
    
    ## @private
    # @return {str}
    def _api_path(self):
        return "/appliance"
    
    ## @private
    # @return {str}
    def _root_key(self):
        return "Appliance"
    
    ## @private
    # @return {str}
    def _root_key_m(self):
        return "Appliances"
    
    ## @private
    # @return {str}
    def _class_name(self):
        return "Appliance"
    
    ## @private
    # @return {str}
    def _id(self):
        return self.get_id()
    
    ## このローカルオブジェクトに現在設定されているリソース情報をAPIに送信し、新規作成または上書き保存します。
    # 
    # @return {saklient.cloud.resources.appliance.Appliance} this
    def save(self):
        return self._save()
    
    ## 最新のリソース情報を再取得します。
    # 
    # @return {saklient.cloud.resources.appliance.Appliance} this
    def reload(self):
        return self._reload()
    
    ## @ignore
    # @param {saklient.cloud.client.Client} client
    # @param {any} obj
    # @param {bool} wrapped=False
    def __init__(self, client, obj, wrapped=False):
        super(Appliance, self).__init__(client)
        Util.validate_type(client, "saklient.cloud.client.Client")
        Util.validate_type(wrapped, "bool")
        self.api_deserialize(obj, wrapped)
    
    ## @private
    # @param {any} query
    # @return {void}
    def _on_before_save(self, query):
        Util.set_by_path(query, "OriginalSettingsHash", self.get_raw_settings_hash())
    
    ## このルータが接続されているスイッチを取得します。
    # 
    # @return {saklient.cloud.resources.swytch.Swytch}
    def get_swytch(self):
        model = Util.create_class_instance("saklient.cloud.models.Model_Swytch", [self._client])
        id = self.get_swytch_id()
        return model.get_by_id(id)
    
    ## アプライアンスの設定を反映します。
    # 
    # @return {saklient.cloud.resources.appliance.Appliance} this
    def apply(self):
        self._client.request("PUT", self._api_path() + "/" + Util.url_encode(self._id()) + "/config")
        return self
    
    ## アプライアンスを起動します。
    # 
    # @return {saklient.cloud.resources.appliance.Appliance} this
    def boot(self):
        self._client.request("PUT", self._api_path() + "/" + Util.url_encode(self._id()) + "/power")
        return self
    
    ## アプライアンスをシャットダウンします。
    # 
    # @return {saklient.cloud.resources.appliance.Appliance} this
    def shutdown(self):
        self._client.request("DELETE", self._api_path() + "/" + Util.url_encode(self._id()) + "/power")
        return self
    
    ## アプライアンスを強制停止します。
    # 
    # @return {saklient.cloud.resources.appliance.Appliance} this
    def stop(self):
        self._client.request("DELETE", self._api_path() + "/" + Util.url_encode(self._id()) + "/power", {
            'Force': True
        })
        return self
    
    ## アプライアンスを強制再起動します。
    # 
    # @return {saklient.cloud.resources.appliance.Appliance} this
    def reboot(self):
        self._client.request("PUT", self._api_path() + "/" + Util.url_encode(self._id()) + "/reset")
        return self
    
    ## 作成中のアプライアンスが利用可能になるまで待機します。
    # 
    # @param {int} timeoutSec=600
    # @return {bool} 成功時はtrue、タイムアウトやエラーによる失敗時はfalseを返します。
    def sleep_while_creating(self, timeoutSec=600):
        Util.validate_type(timeoutSec, "int")
        step = 10
        while (0 < timeoutSec):
            self.reload()
            a = self.get_availability()
            if a == EAvailability.available:
                return True
            if a != EAvailability.migrating:
                timeoutSec = 0
            timeoutSec -= step
            if 0 < timeoutSec:
                Util.sleep(step)
        return False
    
    ## アプライアンスが起動するまで待機します。
    # 
    # @param {int} timeoutSec=600
    # @return {bool}
    def sleep_until_up(self, timeoutSec=600):
        Util.validate_type(timeoutSec, "int")
        return self.sleep_until(EServerInstanceStatus.up, timeoutSec)
    
    ## アプライアンスが停止するまで待機します。
    # 
    # @param {int} timeoutSec=600
    # @return {bool} 成功時はtrue、タイムアウトやエラーによる失敗時はfalseを返します。
    def sleep_until_down(self, timeoutSec=600):
        Util.validate_type(timeoutSec, "int")
        return self.sleep_until(EServerInstanceStatus.down, timeoutSec)
    
    ## アプライアンスが指定のステータスに遷移するまで待機します。
    # 
    # @ignore
    # @param {str} status
    # @param {int} timeoutSec=600
    # @return {bool}
    def sleep_until(self, status, timeoutSec=600):
        Util.validate_type(status, "str")
        Util.validate_type(timeoutSec, "int")
        step = 10
        while (0 < timeoutSec):
            self.reload()
            s = self.get_status()
            if s is None:
                s = EServerInstanceStatus.down
            if s == status:
                return True
            timeoutSec -= step
            if 0 < timeoutSec:
                Util.sleep(step)
        return False
    
    # (instance field) n_id = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_id(self):
        return self.m_id
    
    ## ID
    id = property(get_id, None, None)
    
    # (instance field) n_clazz = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_clazz(self):
        return self.m_clazz
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {str} v
    # @return {str}
    def set_clazz(self, v):
        Util.validate_type(v, "str")
        if not self.is_new:
            raise SaklientException("immutable_field", "Immutable fields cannot be modified after the resource creation: " + "saklient.cloud.resources.appliance.Appliance#clazz")
        self.m_clazz = v
        self.n_clazz = True
        return self.m_clazz
    
    ## クラス {@link EApplianceClass}
    clazz = property(get_clazz, set_clazz, None)
    
    # (instance field) n_name = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_name(self):
        return self.m_name
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {str} v
    # @return {str}
    def set_name(self, v):
        Util.validate_type(v, "str")
        self.m_name = v
        self.n_name = True
        return self.m_name
    
    ## 名前
    name = property(get_name, set_name, None)
    
    # (instance field) n_description = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_description(self):
        return self.m_description
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {str} v
    # @return {str}
    def set_description(self, v):
        Util.validate_type(v, "str")
        self.m_description = v
        self.n_description = True
        return self.m_description
    
    ## 説明
    description = property(get_description, set_description, None)
    
    # (instance field) n_tags = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str[]}
    def get_tags(self):
        self.n_tags = True
        return self.m_tags
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {str[]} v
    # @return {str[]}
    def set_tags(self, v):
        Util.validate_type(v, "list")
        self.m_tags = v
        self.n_tags = True
        return self.m_tags
    
    ## タグ文字列の配列
    tags = property(get_tags, set_tags, None)
    
    # (instance field) n_icon = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {saklient.cloud.resources.icon.Icon}
    def get_icon(self):
        return self.m_icon
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {saklient.cloud.resources.icon.Icon} v
    # @return {saklient.cloud.resources.icon.Icon}
    def set_icon(self, v):
        Util.validate_type(v, "saklient.cloud.resources.icon.Icon")
        self.m_icon = v
        self.n_icon = True
        return self.m_icon
    
    ## アイコン
    icon = property(get_icon, set_icon, None)
    
    # (instance field) n_plan_id = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {int}
    def get_plan_id(self):
        return self.m_plan_id
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {int} v
    # @return {int}
    def set_plan_id(self, v):
        Util.validate_type(v, "int")
        if not self.is_new:
            raise SaklientException("immutable_field", "Immutable fields cannot be modified after the resource creation: " + "saklient.cloud.resources.appliance.Appliance#plan_id")
        self.m_plan_id = v
        self.n_plan_id = True
        return self.m_plan_id
    
    ## プラン
    plan_id = property(get_plan_id, set_plan_id, None)
    
    # (instance field) n_ifaces = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {saklient.cloud.resources.iface.Iface[]}
    def get_ifaces(self):
        return self.m_ifaces
    
    ## インタフェース {@link Iface} の配列
    ifaces = property(get_ifaces, None, None)
    
    # (instance field) n_raw_annotation = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {any}
    def get_raw_annotation(self):
        return self.m_raw_annotation
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {any} v
    # @return {any}
    def set_raw_annotation(self, v):
        if not self.is_new:
            raise SaklientException("immutable_field", "Immutable fields cannot be modified after the resource creation: " + "saklient.cloud.resources.appliance.Appliance#raw_annotation")
        self.m_raw_annotation = v
        self.n_raw_annotation = True
        return self.m_raw_annotation
    
    ## 注釈
    raw_annotation = property(get_raw_annotation, set_raw_annotation, None)
    
    # (instance field) n_raw_settings = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {any}
    def get_raw_settings(self):
        self.n_raw_settings = True
        return self.m_raw_settings
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {any} v
    # @return {any}
    def set_raw_settings(self, v):
        self.m_raw_settings = v
        self.n_raw_settings = True
        return self.m_raw_settings
    
    ## 設定の生データ
    raw_settings = property(get_raw_settings, set_raw_settings, None)
    
    # (instance field) n_raw_settings_hash = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_raw_settings_hash(self):
        return self.m_raw_settings_hash
    
    ## @ignore
    raw_settings_hash = property(get_raw_settings_hash, None, None)
    
    # (instance field) n_status = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_status(self):
        return self.m_status
    
    ## 起動状態 {@link EServerInstanceStatus}
    status = property(get_status, None, None)
    
    # (instance field) n_service_class = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_service_class(self):
        return self.m_service_class
    
    ## サービスクラス
    service_class = property(get_service_class, None, None)
    
    # (instance field) n_availability = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_availability(self):
        return self.m_availability
    
    ## 有効状態 {@link EAvailability}
    availability = property(get_availability, None, None)
    
    # (instance field) n_swytch_id = False
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @return {str}
    def get_swytch_id(self):
        return self.m_swytch_id
    
    ## 接続先スイッチID
    swytch_id = property(get_swytch_id, None, None)
    
    ## (This method is generated in Translator_default#buildImpl)
    # 
    # @param {any} r
    def api_deserialize_impl(self, r):
        self.is_new = r is None
        if self.is_new:
            r = {
                
            }
        self.is_incomplete = False
        if Util.exists_path(r, "ID"):
            self.m_id = None if Util.get_by_path(r, "ID") is None else str(Util.get_by_path(r, "ID"))
        else:
            self.m_id = None
            self.is_incomplete = True
        self.n_id = False
        if Util.exists_path(r, "Class"):
            self.m_clazz = None if Util.get_by_path(r, "Class") is None else str(Util.get_by_path(r, "Class"))
        else:
            self.m_clazz = None
            self.is_incomplete = True
        self.n_clazz = False
        if Util.exists_path(r, "Name"):
            self.m_name = None if Util.get_by_path(r, "Name") is None else str(Util.get_by_path(r, "Name"))
        else:
            self.m_name = None
            self.is_incomplete = True
        self.n_name = False
        if Util.exists_path(r, "Description"):
            self.m_description = None if Util.get_by_path(r, "Description") is None else str(Util.get_by_path(r, "Description"))
        else:
            self.m_description = None
            self.is_incomplete = True
        self.n_description = False
        if Util.exists_path(r, "Tags"):
            if Util.get_by_path(r, "Tags") is None:
                self.m_tags = []
            else:
                self.m_tags = []
                for t in Util.get_by_path(r, "Tags"):
                    v1 = None
                    v1 = None if t is None else str(t)
                    self.m_tags.append(v1)
        else:
            self.m_tags = None
            self.is_incomplete = True
        self.n_tags = False
        if Util.exists_path(r, "Icon"):
            self.m_icon = None if Util.get_by_path(r, "Icon") is None else Icon(self._client, Util.get_by_path(r, "Icon"))
        else:
            self.m_icon = None
            self.is_incomplete = True
        self.n_icon = False
        if Util.exists_path(r, "Plan.ID"):
            self.m_plan_id = None if Util.get_by_path(r, "Plan.ID") is None else int(str(Util.get_by_path(r, "Plan.ID")))
        else:
            self.m_plan_id = None
            self.is_incomplete = True
        self.n_plan_id = False
        if Util.exists_path(r, "Interfaces"):
            if Util.get_by_path(r, "Interfaces") is None:
                self.m_ifaces = []
            else:
                self.m_ifaces = []
                for t in Util.get_by_path(r, "Interfaces"):
                    v2 = None
                    v2 = None if t is None else Iface(self._client, t)
                    self.m_ifaces.append(v2)
        else:
            self.m_ifaces = None
            self.is_incomplete = True
        self.n_ifaces = False
        if Util.exists_path(r, "Remark"):
            self.m_raw_annotation = Util.get_by_path(r, "Remark")
        else:
            self.m_raw_annotation = None
            self.is_incomplete = True
        self.n_raw_annotation = False
        if Util.exists_path(r, "Settings"):
            self.m_raw_settings = Util.get_by_path(r, "Settings")
        else:
            self.m_raw_settings = None
            self.is_incomplete = True
        self.n_raw_settings = False
        if Util.exists_path(r, "SettingsHash"):
            self.m_raw_settings_hash = None if Util.get_by_path(r, "SettingsHash") is None else str(Util.get_by_path(r, "SettingsHash"))
        else:
            self.m_raw_settings_hash = None
            self.is_incomplete = True
        self.n_raw_settings_hash = False
        if Util.exists_path(r, "Instance.Status"):
            self.m_status = None if Util.get_by_path(r, "Instance.Status") is None else str(Util.get_by_path(r, "Instance.Status"))
        else:
            self.m_status = None
            self.is_incomplete = True
        self.n_status = False
        if Util.exists_path(r, "ServiceClass"):
            self.m_service_class = None if Util.get_by_path(r, "ServiceClass") is None else str(Util.get_by_path(r, "ServiceClass"))
        else:
            self.m_service_class = None
            self.is_incomplete = True
        self.n_service_class = False
        if Util.exists_path(r, "Availability"):
            self.m_availability = None if Util.get_by_path(r, "Availability") is None else str(Util.get_by_path(r, "Availability"))
        else:
            self.m_availability = None
            self.is_incomplete = True
        self.n_availability = False
        if Util.exists_path(r, "Switch.ID"):
            self.m_swytch_id = None if Util.get_by_path(r, "Switch.ID") is None else str(Util.get_by_path(r, "Switch.ID"))
        else:
            self.m_swytch_id = None
            self.is_incomplete = True
        self.n_swytch_id = False
    
    ## @ignore
    # @param {bool} withClean=False
    # @return {any}
    def api_serialize_impl(self, withClean=False):
        Util.validate_type(withClean, "bool")
        missing = []
        ret = {
            
        }
        if withClean or self.n_id:
            Util.set_by_path(ret, "ID", self.m_id)
        if withClean or self.n_clazz:
            Util.set_by_path(ret, "Class", self.m_clazz)
        else:
            if self.is_new:
                missing.append("clazz")
        if withClean or self.n_name:
            Util.set_by_path(ret, "Name", self.m_name)
        else:
            if self.is_new:
                missing.append("name")
        if withClean or self.n_description:
            Util.set_by_path(ret, "Description", self.m_description)
        if withClean or self.n_tags:
            Util.set_by_path(ret, "Tags", [])
            for r1 in self.m_tags:
                v = None
                v = r1
                (ret["Tags"] if "Tags" in ret else None).append(v)
        if withClean or self.n_icon:
            Util.set_by_path(ret, "Icon", (None if self.m_icon is None else self.m_icon.api_serialize(withClean)) if withClean else ({
                'ID': "0"
            } if self.m_icon is None else self.m_icon.api_serialize_id()))
        if withClean or self.n_plan_id:
            Util.set_by_path(ret, "Plan.ID", self.m_plan_id)
        else:
            if self.is_new:
                missing.append("plan_id")
        if withClean or self.n_ifaces:
            Util.set_by_path(ret, "Interfaces", [])
            for r2 in self.m_ifaces:
                v = None
                v = (None if r2 is None else r2.api_serialize(withClean)) if withClean else ({
                    'ID': "0"
                } if r2 is None else r2.api_serialize_id())
                (ret["Interfaces"] if "Interfaces" in ret else None).append(v)
        if withClean or self.n_raw_annotation:
            Util.set_by_path(ret, "Remark", self.m_raw_annotation)
        else:
            if self.is_new:
                missing.append("raw_annotation")
        if withClean or self.n_raw_settings:
            Util.set_by_path(ret, "Settings", self.m_raw_settings)
        if withClean or self.n_raw_settings_hash:
            Util.set_by_path(ret, "SettingsHash", self.m_raw_settings_hash)
        if withClean or self.n_status:
            Util.set_by_path(ret, "Instance.Status", self.m_status)
        if withClean or self.n_service_class:
            Util.set_by_path(ret, "ServiceClass", self.m_service_class)
        if withClean or self.n_availability:
            Util.set_by_path(ret, "Availability", self.m_availability)
        if withClean or self.n_swytch_id:
            Util.set_by_path(ret, "Switch.ID", self.m_swytch_id)
        if len(missing) > 0:
            raise SaklientException("required_field", "Required fields must be set before the Appliance creation: " + ", ".join(missing))
        return ret
    



# -*- coding:utf-8 -*-

from ...util import Util
from ..client import Client
from ...errors.httpexception import HttpException
import re

# module saklient.cloud.resources.resource

class Resource:
    ## @ignore
    
    # (instance field) _client
    
    ## @return {saklient.cloud.client.Client}
    def get_client(self):
        return self._client
    
    ## @ignore
    client = property(get_client, None, None)
    
    # (instance field) _query
    
    ## @ignore
    # @param {str} key
    # @param {any} value
    # @return {void}
    def set_param(self, key, value):
        Util.validate_type(key, "str")
        self._query[key] = value
    
    ## @private
    # @return {str}
    def _api_path(self):
        return None
    
    ## @private
    # @return {str}
    def _root_key(self):
        return None
    
    ## @private
    # @return {str}
    def _root_key_m(self):
        return None
    
    ## @private
    # @return {str}
    def _class_name(self):
        return None
    
    ## @private
    # @return {str}
    def _id(self):
        return None
    
    ## @ignore
    # @param {saklient.cloud.client.Client} client
    def __init__(self, client):
        Util.validate_type(client, "saklient.cloud.client.Client")
        self._client = client
        self._query = {}
    
    # (instance field) is_new
    
    # (instance field) is_incomplete
    
    ## @private
    # @param {any} query
    # @return {void}
    def _on_before_save(self, query):
        {}
    
    ## @private
    # @param {any} r
    # @param {any} root
    # @return {void}
    def _on_after_api_deserialize(self, r, root):
        {}
    
    ## @private
    # @param {bool} withClean
    # @return {void}
    def _on_before_api_serialize(self, withClean):
        Util.validate_type(withClean, "bool")
    
    ## @private
    # @param {any} r
    # @param {bool} withClean
    # @return {void}
    def _on_after_api_serialize(self, r, withClean):
        Util.validate_type(withClean, "bool")
    
    ## @ignore
    # @param {any} r
    # @return {void}
    def api_deserialize_impl(self, r):
        {}
    
    ## @ignore
    # @param {any} obj
    # @param {bool} wrapped=False
    # @return {void}
    def api_deserialize(self, obj, wrapped=False):
        Util.validate_type(wrapped, "bool")
        root = None
        record = None
        rkey = self._root_key()
        if obj is not None:
            if not wrapped:
                if rkey is not None:
                    root = {}
                    root[rkey] = obj
                record = obj
            else:
                root = obj
                record = (obj[rkey] if rkey in obj else None)
        self.api_deserialize_impl(record)
        self._on_after_api_deserialize(record, root)
    
    ## @ignore
    # @param {bool} withClean=False
    # @return {any}
    def api_serialize_impl(self, withClean=False):
        Util.validate_type(withClean, "bool")
        return None
    
    ## @ignore
    # @param {bool} withClean=False
    # @return {any}
    def api_serialize(self, withClean=False):
        Util.validate_type(withClean, "bool")
        self._on_before_api_serialize(withClean)
        ret = self.api_serialize_impl(withClean)
        self._on_after_api_serialize(ret, withClean)
        return ret
    
    ## @ignore
    # @return {any}
    def api_serialize_id(self):
        id = self._id()
        if id is None:
            return None
        r = {}
        r["ID"] = id
        return r
    
    ## @ignore
    # @param {str} name
    # @return {str}
    def normalize_field_name(self, name):
        Util.validate_type(name, "str")
        name = re.sub('[A-Z]', lambda m: '_'+m.group(0).lower(), name)
        return name
    
    ## このローカルオブジェクトに現在設定されているリソース情報をAPIに送信し、新規作成または上書き保存します。
    # 
    # @private
    # @return {saklient.cloud.resources.resource.Resource} this
    def _save(self):
        r = self.api_serialize()
        query = self._query
        self._query = {}
        keys = query.keys()
        for k in keys:
            v = (query[k] if k in query else None)
            r[k] = v
        method = "POST" if self.is_new else "PUT"
        path = self._api_path()
        if not self.is_new:
            path += "/" + Util.url_encode(self._id())
        q = {}
        q[self._root_key()] = r
        self._on_before_save(q)
        result = self._client.request(method, path, q)
        self.api_deserialize(result, True)
        return self
    
    ## このローカルオブジェクトのIDと一致するリソースの削除リクエストをAPIに送信します。
    # 
    # @return {void}
    def destroy(self):
        if self.is_new:
            return
        path = self._api_path() + "/" + Util.url_encode(self._id())
        self.request_retry("DELETE", path)
    
    ## 最新のリソース情報を再取得します。
    # 
    # @private
    # @return {saklient.cloud.resources.resource.Resource} this
    def _reload(self):
        result = self.request_retry("GET", self._api_path() + "/" + Util.url_encode(self._id()))
        self.api_deserialize(result, True)
        return self
    
    ## このリソースが存在するかを調べます。
    # 
    # @return {bool}
    def exists(self):
        query = {}
        Util.set_by_path(query, "Filter.ID", [self._id()])
        Util.set_by_path(query, "Include", ["ID"])
        result = self.request_retry("GET", self._api_path(), query)
        cnt = (result["Count"] if "Count" in result else None)
        return cnt == 1
    
    ## @ignore
    # @return {any}
    def dump(self):
        return self.api_serialize(True)
    
    ## @ignore
    # @static
    # @param {str} className
    # @param {saklient.cloud.client.Client} client
    # @param {any} obj
    # @param {bool} wrapped=False
    # @return {saklient.cloud.resources.resource.Resource}
    @staticmethod
    def create_with(className, client, obj, wrapped=False):
        Util.validate_type(className, "str")
        Util.validate_type(client, "saklient.cloud.client.Client")
        Util.validate_type(wrapped, "bool")
        a = [client, obj, wrapped]
        return Util.create_class_instance("saklient.cloud.resources." + className, a)
    
    ## @param {str} method
    # @param {str} path
    # @param {any} query=None
    # @param {int} retryCount=5
    # @param {int} retrySleep=5
    # @return {any}
    def request_retry(self, method, path, query=None, retryCount=5, retrySleep=5):
        Util.validate_type(method, "str")
        Util.validate_type(path, "str")
        Util.validate_type(retryCount, "int")
        Util.validate_type(retrySleep, "int")
        ret = None
        while (1 < retryCount):
            isOk = False
            try:
                ret = self._client.request(method, path, query)
                isOk = True
            except saklient.errors.httpexception.HttpException:
                isOk = False
            if isOk:
                retryCount = -1
            else:
                retryCount -= 1
                Util.sleep(retrySleep)
        if retryCount == 0:
            ret = self._client.request(method, path, query)
        return ret
    



# -*- coding:utf-8 -*-

from ...util import Util

# module saklient.cloud.models.queryparams

class QueryParams:
    ## @ignore
    
    # (instance field) begin
    
    # (instance field) count
    
    # (instance field) filter
    
    # (instance field) sort
    
    def __init__(self):
        self.begin = 0
        self.count = 0
        self.filter = {}
        self.sort = []
    
    ## @return {any}
    def build(self):
        return {
            'From': self.begin,
            'Count': self.count,
            'Filter': self.filter,
            'Sort': self.sort
        }
    



# -*- coding:utf-8 -*-

from ...errors.httppaymentrequiredexception import HttpPaymentRequiredException

# module saklient.cloud.errors.penaltyoperationexception

class PenaltyOperationException(HttpPaymentRequiredException):
    ## お客様のご都合により操作を受け付けることができません。
    
    ## @param {int} status
    # @param {str} code=None
    # @param {str} message=""
    def __init__(self, status, code=None, message=""):
        super(PenaltyOperationException, self).__init__(status, code, "お客様のご都合により操作を受け付けることができません。" if message is None or message == "" else message)
    
